private import std.string;
private import hell2;
private import token;

const int g_fps = 1000/60; // fps

/**
 * 自機
 */
class Player : Token
{
public:
private:
	int   _timer;
public:
	this()
	{
	}
	void create(float x, float y)
	{
		_timer    = 0;
		int s     = CHIP/2;
		super.create(x, y, s, s, s, s);
	}
	void vanish()
	{
		Hell_playWAV("damage");
		Particle.addDestroy(x, y);
		exist = false;
	}
	void update()
	{
		if(isPushEnter())
		{
			Shot s = Shot.add(x+CHIP, y, 0, 0);
			if(s)
			{
				Hell_playWAV("winder");
			}
		}
		if(isPushUp())
		{
			if(y - CHIP > UPPER) y -= CHIP;
		}
		else if(isPushDown())
		{
			if(y + CHIP < BOTTOM) y += CHIP;
		}
		
	}
	void draw()
	{
		if(!exist) return;
		drawAscii(">", x, y);
	}
}

/**
 * 自機のショット
 */
class Shot : Token
{
public:
	const int SIZE_W = CHIP/2;//CHIP;
	const int SIZE_H = CHIP/2;
	const int TIMER_SHOT = 60;
	static TokenManager!(Shot) parent; // 親
	int timer;
public:
	void create(float x, float y, float deg, float speed=0)
	{
		int w      = SIZE_W;
		int h      = SIZE_H;
		super.create(x, y, w, h, w, h, Hell_cosEx(deg)*speed, -Hell_sinEx(deg)*speed);
		timer = TIMER_SHOT;
	}
	static Shot add(float x, float y, float deg, float speed)
	{
		Shot shot = parent.getToken();
		if(!shot) return null;
		shot.create(x, y, deg, speed);
		return shot;
	}
	void vanish()
	{
		exist = false;
	}
	void update()
	{
		timer--;
		if(timer < 1) exist = false;
	}
	void draw()
	{
		if(timer < 30 && timer%8 < 4) return;
		drawAscii("#", x, y);
	}
}

/**
 * アイテム
 */
class Item : Token
{
public:
	static TokenManager!(Item) parent; // 親
	int    id;
	int    timer;
	void create(int id, float x, float y, float deg, float speed=0)
	{
		this.id    = id;
		this.rot   = deg;
		int w      = 8;
		int h      = 8;
		super.create(x, y, w, h, w, h, Hell_cosEx(deg)*speed, -Hell_sinEx(deg)*speed);
	}
	static Item add(int id, float x, float y, float deg, float speed=0)
	{
		Item item = parent.getToken();
		if(item is null) return null;
		item.create(id, x, y, deg, speed);
		return item;
	}
	void vanish()
	{
		exist = false;
	}
	void update()
	{
		super.move();
		if(isOutOfWindow2()) exist = false;
	}
	void draw()
	{
	}
}


/**
 * 敵弾
 */
class Bullet : Token
{
public:
	const int SIZE = CHIP/2;
	int timer;
	int speed;
public:
	static TokenManager!(Bullet) parent; // 親
	void create(float x, float y, float deg=0, float speed=0, float ax=0, float ay=0)
	{
		int s = SIZE;
		super.create(x, y, s, s, s, s, Hell_cosEx(deg)*speed, -Hell_sinEx(deg)*speed, ax, ay);
		rot = deg;
		timer = 0;
		this.speed = cast(int)speed;
	}
	void vanish()
	{
		exist = false;
	}
	static void vanishAll()
	{
		foreach(i, b; parent.getArray())
		{
			if(!b.exist) continue;
			b.vanish();
		}
	}
	static Bullet add(float x, float y, float deg=0, float speed=0, float ax=0, float ay=0)
	{
		Bullet bullet = parent.getToken();
		if(!bullet) return null;
		bullet.create(x, y, deg, speed, ax, ay);
		return bullet;
	}
	void update()
	{
		timer++;
		if(timer >= speed)
		{
			switch(cast(int)rot)
			{
			case 180:
				x -= CHIP;
				break;
			case 135:
				x -= CHIP;
				y -= CHIP;
				break;
			case 225:
				x -= CHIP;
				y += CHIP;
				break;
			}
			timer = 0;
		}
		if(y < UPPER+CHIP/2)  { y = UPPER+CHIP/2;  rot = 225; }
		if(y > BOTTOM-CHIP/2) { y = BOTTOM-CHIP/2; rot = 135; }
		if(x < 0) exist = false;
	}
	void draw()
	{
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		switch(cast(int)rot)
		{
		case 180:
			drawAscii("-", x, y);
			break;
		case 135:
			drawAscii("\\", x, y);
			break;
		case 225:
			drawAscii("/", x, y);
			break;
		}
	}
}

/**
 * 敵
 */
class Enemy : Token
{
public:
	const int SIZE = CHIP/2;
	int id;
	int timer;
public:
	static TokenManager!(Enemy) parent; // 親
	void create(int id, float x, float y, float deg=0, float speed=0)
	{
		int s = SIZE;
		super.create(x, y, s, s, s, s, Hell_cosEx(deg)*speed, -Hell_sinEx(deg)*speed);
		this.id = id;
		timer = 0;
	}
	void vanish()
	{
		if(id == 3)
		{
			exist = false;
			Level.bEnd = true;
			vanishAll();
		}
		Hell_stopWAV(7);
		Hell_playWAV("destroy1", 0, 7);
		Particle.addDestroy(x, y);
		exist = false;
	}
	static void vanishAll()
	{
		foreach(i, b; parent.getArray())
		{
			if(!b.exist) continue;
			b.vanish();
		}
	}
	static int count()
	{
		int ret = 0;
		foreach(e; parent.getArray())
		{
			if(e.exist) ret++;
		}
		return ret;
	}
	static Enemy add(int id, float x, float y, float deg=0, float speed=0)
	{
		Enemy e = parent.getToken();
		if(!e) return null;
		e.create(id, x, y, deg, speed);
		return e;
	}
	static Enemy addEx(int id, int x, int y, float deg=0, float speed=0)
	{
		return add(id, x*CHIP + CHIP/2, HEIGHT/2-CHIP*2+CHIP/2 + y*CHIP, deg, speed);
	}
	void update()
	{
		switch(id)
		{
		case 0:
			if(timer%120 == 0)
			{
				Bullet.add(x, y, 180, 6);
			}
			break;
		case 1:
			if(timer%240 == 0)
			{
				Bullet.add(x, y, 135, 30);
			}
			if(timer%240 == 120)
			{
				Bullet.add(x, y, 225, 30);
			}
			break;
		case 2:
			if(timer%240 == 0)
			{
				Bullet.add(x, y, 135, 10);
			}
			if(timer%240 == 120)
			{
				Bullet.add(x, y, 225, 10);
			}
			if(timer%300 == 60)
			{
				y += Hell_randint(-1, 1)*CHIP;
				if(y < UPPER+CHIP/2)  { y = UPPER+CHIP/2;  }
				if(y > BOTTOM-CHIP/2) { y = BOTTOM-CHIP/2; }
			}
			if(timer%300 == 299)
			{
				Bullet.add(x, y, 180, 7);
			}
			break;
		case 3:
			if(timer%480 == 0)
			{
				Enemy.addEx(4, 24, 0, 180, 1);
			}
			if(timer%480 == 120)
			{
				Enemy.addEx(4, 24, 3, 180, 1);
			}
			if(timer%480 == 240)
			{
				Enemy.addEx(4, 24, 1, 180, 1);
			}
			if(timer%480 == 360)
			{
				Enemy.addEx(4, 24, 2, 180, 1);
			}
			
			if(timer%300 == 60)
			{
				float tmpY = y;
				bool bMove = false;
				while(!bMove)
				{
					bMove = true;
					float tmpY2 = y + Hell_randint(-1, 2)*CHIP;
					if(tmpY2 < UPPER)       { bMove = false; }
					else if(tmpY2 > BOTTOM) { bMove = false; }
					else if(tmpY2 == y)     { bMove = false; }
					else                    { y = tmpY2; bMove = true; }
				}
				Bullet.add(x, tmpY, 180, 12);
				Bullet.add(x, tmpY, 180, 8);
				Bullet.add(x, tmpY, 180, 4);
				Bullet.add(x, y, 180, 10);
				Bullet.add(x, y, 180, 5);
			}
			break;
		case 4:
			break;
		}
		x  += vx; y  += vy;
		timer++;
		if(isOutOfWindow2()) exist = false;
	}
	void draw()
	{
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		switch(id)
		{
		case 0:
		case 4:
			drawAscii("W", x, y);
			break;
		case 1:
			drawAscii("X", x, y);
			break;
		case 2:
			drawAscii("Y", x, y);
			break;
		case 3:
			drawAscii("Z", x, y);
			break;
		}
	}
}

/**
 * 打ち返し弾
 */
class Reflection : Token
{
public:
	const int SIZE_W = CHIP;
	const int SIZE_H = CHIP/2;
public:
	static TokenManager!(Reflection) parent; // 親
	void create(float x, float y, float deg=0, float speed=0)
	{
		int s1 = SIZE_W;
		int s2 = SIZE_H;
		super.create(x, y, s1, s2, s1, s2, Hell_cosEx(deg)*speed, -Hell_sinEx(deg)*speed);
	}
	void vanish()
	{
		exist = false;
	}
	static Reflection add(float x, float y, float deg=0, float speed=0)
	{
		Reflection ref = parent.getToken();
		if(!ref) return null;
		ref.create(x, y, deg, speed);
		ref.rot  = cast(int)deg;
		return ref;
	}
	void update()
	{
		super.move();
		if(isOutOfWindow2()) exist = false;
	}
	void draw()
	{
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		drawAscii(")", x, y);
	}
}

class Back
{
public:
	enum
	{
		STATE_NONE,
		STATE_NORMAL,
		STATE_RASTER,
		STATE_ROT,
		STATE_BLACK_OUT,
		STATE_NOISE,
	}
	const int TIMER_INIT = 120;
	int timer;
	int state;
	int tInit;
public:
	this() { init(); }
	void init()
	{
		timer = 0;
		state = STATE_NORMAL;
		tInit = TIMER_INIT;
	}
	void begin(int state, int timer)
	{
		this.state = state;
		this.timer = timer;
	}
	void update()
	{
		if(tInit > 0) tInit = tInit*90/100;
	}
	void draw()
	{
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		Hell_drawRect(0, 0, WIDTH, HEIGHT, 0, 0, 0x80, 0x80, 0x80);
		
		int h = CHIP*4*(TIMER_INIT-tInit)/TIMER_INIT;
		Hell_drawRectEx(WIDTH/2, HEIGHT/2, WIDTH/2, h/2);
		int y = HEIGHT/2 - CHIP*2;
		for(int i = 0; i < 5; i++)
		{
			int y2 = y+i*CHIP;
			Hell_drawLine(0, y2, WIDTH, y2, 1, 0x80, 0x80, 0x80);
		}
		for(int i = 0; i < WIDTH/CHIP+1; i++)
		{
			int x2 = i*CHIP;
			int y2 = y+4*CHIP;
			Hell_drawLine(x2, y, x2, y2, 1, 0x80, 0x80, 0x80);
		}
	}
}

class Particle : Token
{
public:
	static TokenManager!(Particle) parent; // 親
	char[] id;
	int timer;
	void create(char[] id, float x, float y, int timer, float deg=0, float speed=0)
	{
		super.create(x, y, 0, 0, 0, 0, Hell_cosEx(deg)*speed, -Hell_sinEx(deg)*speed);
		this.id    = id;
		this.timer = timer;
	}
	void vanish()
	{
		exist = false;
	}
	static Particle add(char[] id, float x, float y, int timer, float deg=0, float speed=0)
	{
		Particle p = parent.getToken();
		if(!p) return null;
		p.create(id, x, y, timer, deg, speed);
		return p;
	}
	static void addDestroy(float x, float y)
	{
		float rot = Hell_randint(0, 359);
		for(int i = 0; i < 8; i++)
		{
			add("*", x, y, 60, rot, Hell_randint(4, 5));
			rot += Hell_randint(30, 60);
		}
	}
	void update()
	{
		switch(id)
		{
		case "*":
			vx *= 0.95f;
			vy *= 0.95f;
			break;
		}
		timer--;
		if(timer < 1) exist = false;
		super.move();
		if(isOutOfWindow2()) exist = false;
	}
	void draw()
	{
		Hell_setAlpha(HELL_ALPHA_ADD);
		switch(id)
		{
		case "*":
			if(timer < 30 && timer%8 < 4) return;
			Hell_drawFont(id, x, y, 3f);
			break;
		}
	}
}

class Level
{
public:
	static int    cnt;
	static char[] list;
	static int    timer;
	static bool   bWait;
	static bool   bEnd;
	static void init()
	{
		timer = 0;
		bWait = false;
		bEnd  = false;
		list  = "WWWWWWWWWWWWXXXXWWWWYWWWWYZ";
		cnt   = 0;
	}
	static void del()
	{
		if(list.length > 0)
		{
			list = list[1..$];
		}
	}
	static void update()
	{
		switch(timer)
		{
		case 0:   Enemy.addEx(0, 23, 1, 0, 0); del(); break;
		case 90:  Enemy.addEx(0, 23, 3, 0, 0); del(); break;
		case 150: Enemy.addEx(0, 23, 0, 0, 0); del(); break;
		case 210:
			Enemy.addEx(0, 23, 2, 0, 0);
			del();
			bWait = true;
			timer++;
			break;
		case 240:
			Enemy.addEx(0, 23, 0, 0, 0);
			Enemy.addEx(0, 23, 2, 0, 0);
			del();
			del();
			break;
		case 300:
			Enemy.addEx(0, 23, 1, 0, 0);
			Enemy.addEx(0, 23, 3, 0, 0);
			del();
			del();
			bWait = true;
			timer++;
			break;
		case 330:
			Enemy.addEx(0, 23, 2, 0, 0);
			Enemy.addEx(0, 23, 3, 0, 0);
			del();
			del();
			break;
		case 390:
			Enemy.addEx(0, 23, 0, 0, 0);
			Enemy.addEx(0, 23, 1, 0, 0);
			del();
			del();
			bWait = true;
			timer++;
			break;
		case 420:
			Hell_stopBgm(5000);
			Enemy.addEx(1, 9, 0, 0, 0);
			Enemy.addEx(1, 9, 3, 0, 0);
			del();
			del();
			bWait = true;
			timer++;
			break;
		case 560:
			Hell_playBgm("resource/sound/bgm2.ogg", 0);
			Enemy.addEx(1, 12, 1, 0, 0);
			Enemy.addEx(1, 12, 2, 0, 0);
			del();
			del();
			bWait = true;
			timer++;
			break;
		case 600: Enemy.addEx(0, 24, 1, 180, 1); del(); break;
		case 690: Enemy.addEx(0, 24, 3, 180, 1); del(); break;
		case 750: Enemy.addEx(0, 24, 0, 180, 1); del(); break;
		case 810:
			Enemy.addEx(0, 24, 2, 180, 1);
			del();
			bWait = true;
			timer++;
			break;
		case 840:
			Enemy.addEx(2, 23, 1, 0, 0);
			del();
			bWait = true;
			timer++;
			break;
		case 870:  Enemy.addEx(0, 24, 1, 180, 1); del(); break;
		case 960:  Enemy.addEx(0, 24, 3, 180, 1); del(); break;
		case 1050: Enemy.addEx(0, 24, 0, 180, 1); del(); break;
		case 1140:
			Hell_stopBgm(2000);
			Enemy.addEx(0, 24, 2, 180, 1);
			del();
			bWait = true;
			timer++;
			break;
		case 1170:
			Hell_playBgm("resource/sound/bgm3.ogg", 0);
			Enemy.addEx(2, 19, 3, 0, 0);
			del();
			bWait = true;
			timer++;
			break;
		case 1200:
			Enemy.addEx(3, 24, 1, 0, 0);
			del();
			break;
		case 1240: Enemy.addEx(0, 24, 1, 180, 1); break;
		case 1330: Enemy.addEx(0, 24, 2, 180, 1); break;
		case 1420: Enemy.addEx(0, 24, 0, 180, 1); break;
		case 1510:
			Enemy.addEx(0, 24, 3, 180, 1);
			bWait = true;
			timer++;
			break;
		default:
			break;
		}
		if(bWait)
		{
			bWait = Enemy.count() != 0;
		}
		if(!bWait)
		{
			timer++;
		}
	}
	static void draw()
	{
		if(list.length == 0) return;
		
		cnt++;
		int t = cnt/4;
		if(t >= list.length) cnt = 0;
		foreach(i, c; list)
		{
			int dy = 0;
			if(i%list.length == t-1) dy -= 2;
			if(i%list.length == t)   dy -= 5;
			if(i%list.length == t+1) dy -= 2;
			Hell_drawFont(std.string.toString(c), 100+i*15, 360+dy, 2f);
		}
	}
}

/**
 * メインシーン
 */
class SceneMain
{
public:
	// 定数
	// 状態
	enum {
		STATE_INIT,           // 初期化
		STATE_MAIN,           // メイン
		STATE_HIT_WAIT,       // 敵ヒット
		STATE_DAMAGE_WAIT,    // 自機ダメージ
		STATE_GAMEOVER_INIT,  // ゲームオーバー(初期化)
		STATE_GAMEOVER,       // ゲームオーバー
		STATE_GAMECLEAR_INIT, // ゲームクリアー(初期化)
		STATE_GAMECLEAR,      // ゲームクリアー
	};
	Player       player;          // プレイヤー
	Back         back;            // 背景
	int          state;           // 状態
	int          timer;           // タイマ
	int          tPast;
public:
	this()
	{
		player            = new Player();
		back              = new Back();
		Shot.parent       = new TokenManager!(Shot)(1);
		Reflection.parent = new TokenManager!(Reflection)(32);
		Item.parent       = new TokenManager!(Item)(8);
		Bullet.parent     = new TokenManager!(Bullet)(16);
		Enemy.parent      = new TokenManager!(Enemy)(16);
		Particle.parent   = new TokenManager!(Particle)(256);
	}
	// 初期化
	void init()
	{
		// トークン初期化
		Shot.parent.vanish();
		Reflection.parent.vanish();
		Item.parent.vanish();
		Bullet.parent.vanish();
		Enemy.parent.vanish();
		Particle.parent.vanish();
		Level.init();
		back.init();
		player.create(CHIP/2, UPPER+CHIP/2);
		state    = STATE_INIT;
		timer    = 0;
		tPast    = 0;
	}
	void update()
	{
		Particle.parent.update();
		switch(state)
		{
		case STATE_INIT:
			timer++;
			back.update();
			if(isPushEnter())
			{
				timer = 0;
				state = STATE_MAIN;
				Hell_playBgm("resource/sound/bgm1.ogg", 0);
			}
			break;
		case STATE_MAIN:
			tPast++;
			if(Level.bEnd)
			{
				state = STATE_GAMECLEAR_INIT;
				timer = 30;
				Hell_stopBgm(2000);
			}
			else doMain();
			break;
		case STATE_GAMEOVER_INIT:
			timer--;
			if(timer < 1)
			{
				state = STATE_GAMEOVER;
			}
			break;
		case STATE_GAMEOVER:
			if(isPushEnter())
			{
				init();
			}
			break;
		case STATE_GAMECLEAR_INIT:
			timer--;
			if(timer < 1)
			{
				state = STATE_GAMECLEAR;
			}
			break;
		case STATE_GAMECLEAR:
			if(isPushEnter())
			{
				init();
			}
			break;
		}
	}
	void doMain()
	{
		back.update();
		player.update();
		Shot.parent.update();
		Reflection.parent.update();
		Item.parent.update();
		Bullet.parent.update();
		Enemy.parent.update();
		Level.update();
		foreach(e; Enemy.parent.getArray())
		{
			if(!e.exist) continue;
			foreach(ref; Reflection.parent.getArray())
			{
				if(!ref.exist)    continue;
				if(!ref.isHit(e)) continue;
				e.vanish();
				ref.vanish();
			}
		}
		foreach(bullet; Bullet.parent.getArray())
		{
			if(!bullet.exist) continue;
			foreach(shot; Shot.parent.getArray())
			{
				if(!shot.exist)         continue;
				if(!shot.isHit(bullet)) continue;
				Reflection.add(bullet.x, bullet.y, 0, 20);
				bullet.vanish();
			}
			if(!bullet.exist) continue;
			if(player.isHit(bullet))
			{
				Hell_stopBgm(2000);
				state = STATE_GAMEOVER_INIT;
				timer = 30;
				player.vanish();
			}
		}
	}
	void draw()
	{
		back.draw();
		player.draw();
		Shot.parent.draw();
		Reflection.parent.draw();
		Enemy.parent.draw();
		Bullet.parent.draw();
		Particle.parent.draw();
		switch(state)
		{
		case STATE_INIT:
			Hell_drawFont("RECT WINDER <ASCII>", 320, 200, 2f);
			if(timer%120 < 90)
			{
				Hell_drawFont("+PUSH SPACE+", 320, 280, 2f);
			}
			break;
		case STATE_GAMEOVER:
			Hell_drawFont("GAME OVER", 320, 240, 2f);
			break;
		case STATE_GAMECLEAR:
			Hell_drawFont("GAME CLEAR", 320, 240, 2f);
			break;
		default:
			break;
		}
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		int min  = tPast/60/60;
		int sec  = (tPast/60)%60;
		int msec = (tPast%60) * 1000 / 60;
		Hell_drawFontEx(320, 100, 4f, 0xff, 0xff, 0xff, 0xff, "TIME %02d:%02d:%03d", min, sec, msec);
		Level.draw();
	}
}

SceneMain gSceneMain;
// ゲーム初期化
void init()
{
	// リソース読み込み
	Hell_loadWAV("winder", "resource/sound/winder.wav");
	Hell_loadWAV("damage", "resource/sound/damage.wav");
	Hell_loadWAV("destroy1", "resource/sound/destroy1.wav");
	gSceneMain  = new SceneMain();
	gSceneMain.init();
}

// ゲームループ
void loop()
{
	gSceneMain.update();
	gSceneMain.draw();
}

// ユーティリティ
void drawAscii(char[] ascii, float x, float y)
{
	int px = ((cast(int)x)/CHIP) * CHIP + CHIP/2;
	if(y < 188) return;
	if(y > 292) return;
	int py = 0;
	if(188 < y && y <= 214) py = 188;
	if(214 < y && y <= 240) py = 214;
	if(240 < y && y <= 266) py = 240;
	if(266 < y && y <= 292) py = 266;
	py += CHIP/2;
	Hell_drawFont(ascii, px, py, 3f);
}

bool isPushUp()
{
	if(Hell_isPushKey(HELL_UP))    return true;
	if(Hell_isPushJKey(HELL_J_UP)) return true;
	return false;
}

bool isPushDown()
{
	if(Hell_isPushKey(HELL_DOWN))    return true;
	if(Hell_isPushJKey(HELL_J_DOWN)) return true;
	return false;
}

bool isPressEnter()
{
	if(Hell_isPressKey(HELL_z)) return true;
	if(Hell_isPressJButton(0))  return true;
	if(Hell_isPressMouse() & HELL_BUTTON_LEFT) return true;
	return false;
}
bool isPushEnter()
{
	if(Hell_isPushKey(HELL_z))     return true;
	if(Hell_isPushKey(HELL_SPACE)) return true;
	if(Hell_isPushJButton(0))      return true;
	if(Hell_isPushMouse() & HELL_BUTTON_LEFT) return true;
	return false;
}
bool isPressCancel()
{
	if(Hell_isPressKey(HELL_x)) return true;
	if(Hell_isPressJButton(1))  return true;
	if(Hell_isPressMouse() & HELL_BUTTON_RIGHT) return true;
	return false;
}
bool isPushCancel()
{
	if(Hell_isPushKey(HELL_x)) return true;
	if(Hell_isPushJButton(1))  return true;
	if(Hell_isPushMouse() & HELL_BUTTON_RIGHT) return true;
	return false;
}
bool isPushMenu()
{
	if(Hell_isPushKey(HELL_c)) return true;
	if(Hell_isPushJButton(2))  return true;
	if(Hell_isPushMouse() & HELL_BUTTON_MIDDLE) return true;
	return false;
}

int Hell_main(char[][] args)
{
	try
	{
		// 画面を作る
		if(args.length > 1)
		{
			Hell_init("RectWinder@", WIDTH, HEIGHT, true);
		}
		else
		{
			Hell_init("RectWinder@", WIDTH, HEIGHT);
		}
		init();
		while(true)
		{
			Hell_begin();
			loop();
			Hell_drawFPS(WIDTH/2, HEIGHT-8/2*2-2);
			Hell_update();
			Hell_wait(g_fps);
		}
	}
	catch(Exception e)
	{
		Hell_write(e);
	}
	finally
	{
		Hell_quit();
	}
	
	return 0;
}


public int main(char[][] args)
{
	return Hell_main(args);
}

