private import hell2;

const int WIDTH = 640;
const int HEIGHT = 480;
const int CHIP = 26;
const int UPPER  = HEIGHT/2-CHIP*2;
const int BOTTOM = HEIGHT/2+CHIP*2;


/**
 * トークンクラス
 */
class Token
{
public:
	float x, y;         // 座標
	float prvX, prvY;   // 前回の座標
	float prv2X, prv2Y; // 前々回の座標
	float vx, vy;       // 移動量
	float ax, ay;       // 加速度
	bool  exist;        // 生存フラグ
	float w, h;         // サイズ
	float gw, gh;       // 表示サイズ
	float rot;          // 回転角度
public:
	/**
	 * コンストラクタ
	 */
	this()
	{
		exist = false;
		rot   = 0f;
	}
	/**
	 * 生成
	 */
	void create(float x, float y, float w, float h, float gw, float gh, float vx=0f, float vy=0f, float ax=0f, float ay=0f)
	{
		prv2X = x; prv2Y = y;
		prvX  = x; prvY  = y;
		this.x  = x;  this.y  = y;
		this.w  = w;  this.h  = h;
		this.gw = gw; this.gh = gh;
		this.vx = vx; this.vy = vy;
		this.ax = ax; this.ay = ay;
		exist = true;
	}
	/**
	 * 移動
	 */
	void move()
	{
		prv2X = prvX; prv2Y = prvY;
		prvX  = x;    prvY  = y;
		vx += ax; vy += ay;
		x += vx;  y += vy;
	}
	/**
	 * 点との当たり判定
	 */
	bool isHit(int px, int py)
	{
		return x <= px && px <= x + w && y <= py && py <= y + h;
	}
	/**
	 * 別のトークン（矩形）との当たり判定
	 */
	bool isHit(Token t)
	{
		float left1   = x - w;
		float top1    = y - h;
		float right1  = x + w;
		float bottom1 = y + h;
		float left2   = t.x - t.w;
		float top2    = t.y - t.h;
		float right2  = t.x + t.w;
		float bottom2 = t.y + t.h;
		return (left1 < right2) && (right1 > left2) && (top1 < bottom2) && (bottom1 > top2);
	}
	/**
	 * 別のトークン（円）との当たり判定
	 */
	bool isHit2(Token t)
	{
		float dx = t.x - x;
		float dy = t.y - y;
		return (dx*dx + dy*dy < t.w*t.w + w*w);
	}
	/**
	 * 別のトークン（矩形）との当たり判定（グラフィックサイズで判定）
	 */
	bool isHitGraphic(Token t)
	{
		float left1   = x - gw;
		float top1    = y - gh;
		float right1  = x + gw;
		float bottom1 = y + gh;
		float left2   = t.x - t.gw;
		float top2    = t.y - t.gh;
		float right2  = t.x + t.gw;
		float bottom2 = t.y + t.gh;
		return (left1 < right2) && (right1 > left2) && (top1 < bottom2) && (bottom1 > top2);
	}
	/**
	 * ウィンドウサイズ内に存在するかどうか
	 */
	bool isOutOfWindow()
	{
		return x < -w/2 || x-w/2 > WIDTH || y < -h/2 || y-h/2 > HEIGHT;
	}
	bool isOutOfWindow2()
	{
//		return x < -w || x-w > WIDTH || y < -h || y-h > HEIGHT;
		return x < -w || x-w > WIDTH || y < -h+UPPER || y-h > BOTTOM;
	}
	void dump()
	{
		printf("(x,y)=(%f,%f)\n", x, y);
	}
	/**
	 * 描画
	 */
	void draw(int r, int g, int b)
	{
		Hell_drawRectEx(cast(int)x, cast(int)y, cast(int)w/2, cast(int)h/2, 0, cast(int)rot, r, g, b);
	}
}

/**
 * トークンの管理クラス
 */
class TokenManager(T)
{
public:
	T[] _pool; // 保持しているインスタンス
public:
	/**
	 * コンストラクタ
	 */
	this(int size)
	{
		_pool = new T[size];
		foreach(inout t; _pool) { t = new T; }
	}
	/**
	 * 利用可能なトークンの取得
	 */
	T getToken()
	{
		foreach(t; _pool) { if(!t.exist) return t; }
		return null;
	}
	/**
	 * 保持しているトークン配列の取得
	 */
	T[] getArray() { return _pool; }
	/**
	 * 生存数を返す
	 */
	int count()
	{
		int result = 0;
		foreach(t; _pool) { if(t.exist) result++; }
		return result;
	}
	/**
	 * 更新
	 */
	void update()
	{
		foreach(t; _pool)
		{
			if(!t.exist) continue;
			t.update();
		}
	}
	/**
	 * 描画
	 */
	void draw()
	{
		foreach(t; _pool)
		{
			if(!t.exist) continue;
			t.draw();
		}
	}
	/**
	 * 全てを消滅
	 */
	void vanish()
	{
		foreach(t; _pool) { t.exist = false; }
	}
}
